"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""

import pandas as pd


class Acs(object):

    __DATAPATH__ = '%s/df_AcsRead%s_%s_%d_%d.csv'

    def __init__(self, data_path='data/data_output'):
        self.datapath = data_path

    def _get_df_(self, datapath, normed=False):
        df = pd.read_csv(datapath)
        df = df.set_index('SPATIAL')
        column_total = df.columns[df.columns.str.endswith('_TOTAL')][0]
        if normed:
            df.loc[:, df.columns != column_total] = df.loc[:, df.columns != column_total].apply(lambda r: r / df[column_total], axis=0)
        df = df.dropna()
        return df

    def get_race(self, level, year, estimate, normed=False):
        datapath = self.__DATAPATH__ % (self.datapath, 'Race', level, year, estimate)
        df = self._get_df_(datapath, normed)
        return df

    def get_education(self, level, year, estimate, normed=False):
        datapath = self.__DATAPATH__ % (self.datapath, 'Education', level, year, estimate)
        df = self._get_df_(datapath, normed)
        return df

    def get_income(self, level, year, estimate, normed=False):
        datapath = self.__DATAPATH__ % (self.datapath, 'Income', level, year, estimate)
        df = self._get_df_(datapath, normed)
        return df

    def get_acs(self, level, year, estimate, normed=False):
        # Read individual files
        df_race = self.get_race(level, year, estimate, normed)
        df_education = self.get_education(level, year, estimate, normed)
        df_income = self.get_income(level, year, estimate, normed)
        # Merge pairs of files
        df = pd.merge(df_race, df_education, left_index=True, right_index=True)
        df = pd.merge(df, df_income, left_index=True, right_index=True)
        return df

    # def get_implementations(self):
    #     dict_area_function = {
    #         'race': None,
    #         'education': None,
    #         'income': None
    #     }
    #     return dict_area_function
    #
    #
    # def get_data(self, data_name):
    #         dict_data_function = self.get_implementations()
    #         assert data_name in dict_data_function, 'area not implemented'
    #         data = dict_data_function[data_name]
    #         method_to_call = getattr(data(), 'get_data')
    #         return method_to_call()



__author__ = 'diegopinheiro'
__email__ = 'diegompin@gmail.com'
__github__ = 'https://github.com/diegompin'

from acs_wrapper.src.read.acs_read_base import AcsRead


class AcsReadTypeHealthInsuranceAge(AcsRead):

    def __init__(self, datapath):
        super().__init__(datapath=datapath, prefix='INSURANCECOREVAGE', datafile='B27010',
                         column_format='HD01_VD%02d', is_percent=False)

    def get_mappings(self):
        '''
        HD01_VD01
        Estimate; Total:

        HD01_VD04
        Estimate; Under 19 years:
        - With one type of health insurance coverage:
        - With employer-based health insurance only

        HD01_VD05
        Estimate; Under 19 years:
        - With one type of health insurance coverage:
        - With direct-purchase health insurance only

        HD01_VD06
        Estimate; Under 19 years:
        - With one type of health insurance coverage:
        - With Medicare coverage only

        HD01_VD07
        Estimate; Under 19 years:
        - With one type of health insurance coverage:
        - With Medicaid/means-tested public coverage only

        HD01_VD08
        Estimate; Under 19 years:
        - With one type of health insurance coverage:
        - With TRICARE/military health coverage only

        HD01_VD09
        Estimate; Under 19 years:
        - With one type of health insurance coverage:
        - With VA Health Care only

        HD01_VD11
        Estimate; Under 19 years:
        - With two or more types of health insurance coverage:
        - With employer-based and direct-purchase coverage

        HD01_VD12
        Estimate; Under 19 years:
        - With two or more types of health insurance coverage:
        - With employer-based and Medicare coverage

        HD01_VD13
        Estimate; Under 19 years:
        - With two or more types of health insurance coverage:
        - With Medicare and Medicaid/means-tested public coverage

        HD01_VD14
        Estimate; Under 19 years:
        - With two or more types of health insurance coverage:
        - Other private only combinations

        HD01_VD15
        Estimate; Under 19 years:
        - With two or more types of health insurance coverage:
        - Other public only combinations

        HD01_VD16
        Estimate; Under 19 years:
        - With two or more types of health insurance coverage:
        - Other coverage combinations

        HD01_VD17
        Estimate; Under 19 years:
        - No health insurance coverage

        HD01_VD20
        Estimate; 19 to 34 years:
        - With one type of health insurance coverage:
        - With employer-based health insurance only

        HD01_VD21
        Estimate; 19 to 34 years:
        - With one type of health insurance coverage:
        - With direct-purchase health insurance only

        HD01_VD22
        Estimate; 19 to 34 years:
        - With one type of health insurance coverage:
        - With Medicare coverage only

        HD01_VD23
        Estimate; 19 to 34 years:
        - With one type of health insurance coverage:
        - With Medicaid/means-tested public coverage only

        HD01_VD24
        Estimate; 19 to 34 years:
        - With one type of health insurance coverage:
        - With TRICARE/military health coverage only

        HD01_VD25
        Estimate; 19 to 34 years:
        - With one type of health insurance coverage:
        - With VA Health Care only

        HD01_VD27
        Estimate; 19 to 34 years:
        - With two or more types of health insurance coverage:
        - With employer-based and direct-purchase coverage

        HD01_VD28
        Estimate; 19 to 34 years:
        - With two or more types of health insurance coverage:
        - With employer-based and Medicare coverage

        HD01_VD29
        Estimate; 19 to 34 years:
        - With two or more types of health insurance coverage:
        - With Medicare and Medicaid/means-tested public coverage

        HD01_VD30
        Estimate; 19 to 34 years:
        - With two or more types of health insurance coverage:
        - Other private only combinations

        HD01_VD31
        Estimate; 19 to 34 years:
        - With two or more types of health insurance coverage:
        - Other public only combinations

        HD01_VD32
        Estimate; 19 to 34 years:
        - With two or more types of health insurance coverage:
        - Other coverage combinations

        HD01_VD33
        Estimate; 19 to 34 years:
        - No health insurance coverage

        HD01_VD36
        Estimate; 35 to 64 years:
        - With one type of health insurance coverage:
        - With employer-based health insurance only

        HD01_VD37
        Estimate; 35 to 64 years:
        - With one type of health insurance coverage:
        - With direct-purchase health insurance only

        HD01_VD38
        Estimate; 35 to 64 years:
         With one type of health insurance coverage:
         - With Medicare coverage only

        HD01_VD39
        Estimate; 35 to 64 years:
        - With one type of health insurance coverage:
        - With Medicaid/means-tested public coverage only

        HD01_VD40
        Estimate; 35 to 64 years:
        - With one type of health insurance coverage:
        - With TRICARE/military health coverage only

        HD01_VD41
        Estimate; 35 to 64 years:
        - With one type of health insurance coverage:
        - With VA Health Care only

        HD01_VD43
        Estimate; 35 to 64 years:
        - With two or more types of health insurance coverage:
        - With employer-based and direct-purchase coverage

        HD01_VD44
        Estimate; 35 to 64 years:
        - With two or more types of health insurance coverage:
        - With employer-based and Medicare coverage

        HD01_VD45
        Estimate; 35 to 64 years:
        - With two or more types of health insurance coverage:
        - With direct-purchase and Medicare coverage

        HD01_VD46
        Estimate; 35 to 64 years:
        - With two or more types of health insurance coverage:
        - With Medicare and Medicaid/means-tested public coverage

        HD01_VD47
        Estimate; 35 to 64 years:
        - With two or more types of health insurance coverage:
        - Other private only combinations

        HD01_VD48
        Estimate; 35 to 64 years:
        - With two or more types of health insurance coverage:
        - Other public only combinations

        HD01_VD49
        Estimate; 35 to 64 years:
        - With two or more types of health insurance coverage:
        - Other coverage combinations

        HD01_VD50
        Estimate; 35 to 64 years:
        - No health insurance coverage

        HD01_VD53
        Estimate; 65 years and over:
        - With one type of health insurance coverage:
        - With employer-based health insurance only
        HD01_VD54
        Estimate; 65 years and over:
        - With one type of health insurance coverage:
        - With direct-purchase health insurance only

        HD01_VD55
        Estimate; 65 years and over:
        - With one type of health insurance coverage:
        - With Medicare coverage only

        HD01_VD56
        Estimate; 65 years and over:
        - With one type of health insurance coverage:
        - With TRICARE/military health coverage only

        HD01_VD57
        Estimate; 65 years and over:
        - With one type of health insurance coverage:
        - With VA Health Care only

        HD01_VD59
        Estimate; 65 years and over:
        - With two or more types of health insurance coverage:
        - With employer-based and direct-purchase coverage

        HD01_VD60
        Estimate; 65 years and over:
        - With two or more types of health insurance coverage:
        - With employer-based and Medicare coverage

        HD01_VD61
        Estimate; 65 years and over:
        - With two or more types of health insurance coverage:
        - With direct-purchase and Medicare coverage

        HD01_VD62
        Estimate; 65 years and over:
        - With two or more types of health insurance coverage:
        - With Medicare and Medicaid/means-tested public coverage

        HD01_VD63
        Estimate; 65 years and over:
        - With two or more types of health insurance coverage:
        - Other private only combinations

        HD01_VD64
        Estimate; 65 years and over:
        - With two or more types of health insurance coverage:
        - Other public only combinations

        HD01_VD65
        Estimate; 65 years and over:
        - With two or more types of health insurance coverage:
        - Other coverage combinations

        HD01_VD66
        Estimate; 65 years and over:
        - No health insurance coverage



        '''
        mappings = [
            {
                'TOTAL': [1],
                'EMPLOYER_BASED': [4, 20, 36, 53],
                'DIRECT_PURCHASE': [5, 14, 21, 37, 54],
                'EMPLOYER_BASED_DIRECT_PURCHASE': [11, 27, 43, 59],
                'EMPLOYER_BASED_MEDICARE': [12, 28, 44, 60],
                'DIRECT_PURCHASE_MEDICARE': [45, 61],
                'MEDICARE': [6, 22, 38, 55],
                'MEDICAID': [7, 23, 39],
                'MEDICARE_MEDICAID': [13, 29, 46, 62],
                'TRICARE_MILITARY': [8, 24, 40, 56],
                'VETERAN': [9, 25, 41, 67],
                'OTHER': [14, 15, 16, 30, 31, 32, 47, 48, 49, 63, 64, 65],
                'NO_INSURANCE': [17, 33, 50, 66],

            }
        ]
        return mappings



"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""

from acs_wrapper.src.read import *


class AcsPreprocess(object):

    def __init__(self):
        pass

    @staticmethod
    def get_param():
        readers = ALL_IMPLEMENTATIONS
        # readers = [
        #     AcsReadRace,
        #     AcsReadEducation,
        #     AcsReadIncome,
        #     AcsReadAge65,
        #
        # ]
        # readers = [AcsReadRace]
        # level = ['county', 'zcta']
        level = ['zcta']

        year = [17]
        estimates = [5]
        return readers, level, year, estimates



"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""

from old.common import ScriptBase
from acs_wrapper.acs import Acs
import pandas as pd
import itertools as it


class ScriptAcs(ScriptBase):
    def __init__(self, par):
        super().__init__(par)

    def get_file_output(self, params):
        (df, xvar, yvar) = params
        data_link_output = self.data_link_output
        file_output = '%s/US/%s__%s__%s.pdf' % (data_link_output, self.name, xvar, yvar)
        print(file_output)
        return file_output

    def get_params(self):

        estimate = 5
        year = 16
        level = 'county'

        acs = Acs()
        normed = True
        df_race = acs.get_race(level, year, estimate, normed=normed)
        df_education = acs.get_education(level, year, estimate, normed=normed)
        df_income = acs.get_income(level, year, estimate, normed=normed)
        df = pd.merge(df_race, df_education, left_index=True, right_index=True)
        df = pd.merge(df, df_income, left_index=True, right_index=True)
        cols = [c for c in df.columns if not c.endswith('_TOTAL')]
        df = df[cols]
        # cols = [c.split('_')[1].title() for c in cols]
        # df.columns = cols
        # mapping_income = {'A': '10k', 'B': '10k-15k', 'C': '15k-25k', 'D': '25k-35k',
        #                   'E': '35k-50k', 'F': '50k-75k', 'G': '75k-100k', 'H': '100k-150k', 'I': '150k-200k',
        #                   'J': '200k'}
        # df = df.rename(columns=mapping_income)

        # return [(df, c1, c2) for (c1,c2) in it.combinations(df.columns[0:3], r=2)]
        return it.product([df], df.columns, [c for c in df.columns if c.endswith('ELEMENTAR')])
        # return it.product([df], df.columns, df.columns)




"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""
import pandas as pd
import itertools


class AcsRead(object):
    # __COLUMN_GEO__ = 'GEO.display-label'
    __COLUMN_GEO__ = 'GEO.id2'
    __ENCODING__ = "ISO-8859-1"

    # def __init__(self, datapath, prefix, datafile, year=2016, estimates=5, column_format='HC01_EST_VC%02d', is_percent=False):
    def __init__(self, datapath, prefix, datafile, column_format='HC01_EST_VC%02d',
                     is_percent=False):
        self.datapath = datapath
        self.prefix = prefix
        self.datafile = datafile
        # self.year = year
        # self.estimates = estimates
        self.column_format = column_format
        self.is_percent = is_percent

    def get_mappings(self):
        """
        Each class should override this method with the respective mapping
        :return:
        """
        pass

    def get_data(self, level, year, estimates):
        df = self.read_file(level, year, estimates)
        df = self._extract_group_mappings(df)
        df = self._filter_columns(df)
        return df

    # def read_file(self):
    def read_file(self, level, year, estimates):
        df = pd.read_csv(self._get_filename(level, year, estimates), encoding=self.__ENCODING__)
        # Eliminates the first line which is a description of the colunm
        df = df.loc[1:, :]
        # Extract only the zip code such as '00601' from 'ZCTA5 00601'
        # df.loc[:, 'SPATIAL'] = df.loc[:, self.__COLUMN_GEO__].apply(lambda cell: cell if cell.split(' ')[0] != 'ZCTA5' else cell.split(' ')[1] )
        df.loc[:, 'SPATIAL'] = df.loc[:, self.__COLUMN_GEO__]
        df['SPATIAL'] = df['SPATIAL'].astype('str')
        # Set the index
        df = df.set_index('SPATIAL')
        return df

    def _get_column_name(self, columns):
        if not isinstance(columns, list):
            column_format = self.column_format
            return [column_format % columns]
        columns_list = [self._get_column_name(c) for c in columns]
        return list(itertools.chain(*columns_list))

    def _extract_group_mappings(self, df):
        mappings = self.get_mappings()
        assert mappings is not None, 'No mapping provided'
        df = df.copy(deep=True)
        # TODO Generalize file output
        for m in mappings[0]:
            df.loc[:, m] = 0
        for group_mapping in mappings:
            df_group = df.copy(deep=True)
            for m in group_mapping:
                columns_group = self._get_column_name(group_mapping[m])
                # Summing the columns belonging to the same group
                df_group.loc[:, m] = df_group.loc[:, columns_group].apply(pd.to_numeric, errors='coerce').sum(axis=1)
            # Multiply the percentages by the total
            if self.is_percent:
                # Divide by 100 the percentages in df_group (i.e., 45.6 -> .456) and multiply by the total
                columns = list(mappings[0].keys())
                df_group = df_group[columns]
                columns_except_total = df_group.columns != 'TOTAL'
                df_group.loc[:, columns_except_total] = (df_group.loc[:, columns_except_total] / 100).multiply(
                    df_group['TOTAL'].values, axis=0)

            for m in group_mapping:
                df.loc[:, m] += df_group.loc[:, m]
        return df

    def _filter_columns(self, df):
        mappings = self.get_mappings()
        prefix = self.prefix
        # Filter only the columns of interest, i.e., mapping.keys()
        df = df[list(mappings[0].keys())]
        # Rename the columns by adding the prefix, e.g. WHITE -> RAC_WHITE
        df.columns = ['%s_%s' % (prefix, c) for c in df.columns]
        return df

    def _get_filename(self, level, year, estimates):
        # return 'datalink/acs_wrapper/data_input/ACS_%d_%dYR_%s_with_ann.csv' % (self.year, self.estimates, self.datafile)
        return f'{self.datapath}/{level}/ACS_{year}_{estimates}YR_{self.datafile}_with_ann.csv'




__author__ = 'diegopinheiro'
__email__ = 'diegompin@gmail.com'
__github__ = 'https://github.com/diegompin'

from acs_wrapper.src.read.acs_read_base import AcsRead


class AcsReadIncome(AcsRead):

    def __init__(self, datapath):
        super().__init__(datapath=datapath, prefix='INC', datafile='S1901', column_format='HC01_EST_VC%02d',
                         is_percent=True)

    def get_mappings(self):
        """
        HC01_EST_VC02
        Households; Estimate; Less than $10,000

        HC01_EST_VC03
        Households; Estimate; $10,000 to $14,999

        HC01_EST_VC04
        Households; Estimate; $15,000 to $24,999

        HC01_EST_VC05
        Households; Estimate; $25,000 to $34,999

        HC01_EST_VC06
        Households; Estimate; $35,000 to $49,999

        HC01_EST_VC07
        Households; Estimate; $50,000 to $74,999

        HC01_EST_VC08
        Households; Estimate; $75,000 to $99,999

        HC01_EST_VC09
        Households; Estimate; $100,000 to $149,999

        HC01_EST_VC10
        Households; Estimate; $150,000 to $199,999

        HC01_EST_VC11
        Households; Estimate; $200,000 or more
        """
        mappings = [{
            'TOTAL': 1,
            'A': 2,
            'B': 3,
            'C': 4,
            'D': 5,
            'E': 6,
            'F': 7,
            'G': 8,
            'H': 9,
            'I': 10,
            'J': 11
        }]
        return mappings



"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""

from acs_wrapper import Acs
import pandas as pd

def main(datalink_acs='datalink/acs_wrapper', level='zcta'):
    df_rac = pd.read_csv('%s/data_output/df_AcsReadRace_%s_16_5.csv' % (datalink_acs, level), index_col=0)
    df_edu = pd.read_csv('%s/data_output/df_AcsReadEducation_%s_16_5.csv' % (datalink_acs, level), index_col=0)
    df_inc = pd.read_csv('%s/data_output/df_AcsReadIncome_%s_16_5.csv' % (datalink_acs, level), index_col=0)
    df_rac = df_rac.loc[:, df_rac.columns != 'RAC_TOTAL'].apply(lambda r: r / df_rac['RAC_TOTAL'], axis=0)
    df_edu = df_edu.loc[:, df_edu.columns != 'EDU_TOTAL'].apply(lambda r: r / df_edu['EDU_TOTAL'], axis=0)
    df_inc = df_inc.loc[:, df_inc.columns != 'INC_TOTAL'].apply(lambda r: r / df_inc['INC_TOTAL'], axis=0)
    # join
    df = df_rac.join(df_edu)
    df = df.join(df_inc)
    df = df.dropna(axis=0)
    # TODO generalize
    # filename_output = 'acs_wrapper/data_output/acs_%d_%d.hdf' % (16, 5)
    filename_output = 'datalink/acs_wrapper/data_output/acs_%d_%d_%s.csv' % (16, 5, level)
    df.to_csv(filename_output)


if __name__ == "__main__":
    main()


# datalink_acs='datalink/acs_wrapper'
# df_rac = pd.read_csv('%s/data_output/df_AcsReadRace_county_16_5.csv' % datalink_acs)
# df_edu = pd.read_csv('%s/data_output/df_AcsReadEducation_county_16_5.csv' % datalink_acs)
# df = df_rac.merge(df_edu, how='inner', left_index=True, right_index=True)


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="acs_wrapper",
    version="1.0.16",
    author="Diego Pinheiro",
    author_email="diegompin@gmail.com",
    description="A wrapper for the American Community Survey Data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/diegompin/american-community-survey",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    install_requires=[
          'pandas',
      ],
)


"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""
import multiprocessing
import multiprocessing.pool

class NoDaemonProcess(multiprocessing.Process):
    # make 'daemon' attribute always return False
    def _get_daemon(self):
        return False
    def _set_daemon(self, value):
        pass
    daemon = property(_get_daemon, _set_daemon)

class MyPool(multiprocessing.pool.Pool):
    Process = NoDaemonProcess


"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""

from acs_wrapper.src.parameters import Parameter
from acs_wrapper.src.util_multiprocessing import MyPool


class ScriptBase(object):
    PAR_SCRIPT_NAME = 'script.name'
    PAR_SCRIPT_PROCESSES = 'script.processes'
    PAR_SCRIPT_DATA_LINK_OUTPUT = 'script.data_link_output'
    PAR_SCRIPT_DATA_LINK_INPUT = 'script.data_link_input'
    PAR_SCRIPT_FILE_FORMAT = 'script.plot_format'


    def __init__(self, par):
        self.pool = MyPool
        self.par = par
        self.name = Parameter.get_par(par, ScriptBase.PAR_SCRIPT_NAME, 'unnamed')
        self.data_link_input = Parameter.get_par(par, ScriptBase.PAR_SCRIPT_DATA_LINK_INPUT, '.')
        self.data_link_output = Parameter.get_par(par, ScriptBase.PAR_SCRIPT_DATA_LINK_OUTPUT, '.')
        self.params = self.get_params()
        self.processes = Parameter.get_par(par, ScriptBase.PAR_SCRIPT_PROCESSES, 4)
        self.file_format = Parameter.get_par(par, ScriptBase.PAR_SCRIPT_FILE_FORMAT, 'PDF')

    def map(self, func, params):
        with self.pool(processes=self.processes) as pool:
            pool.map(func, params)

    def get_params(self):
        pass

    def get_func(self, params):
        pass
        # data_link_output = self.data_link_output
        # filename = self.get_filename(params)
        # file_output = '%s/%s.pdf' % (data_link_output, filename)
        # print(file_output)

    def get_file_output(self, params):
        pass

    def main(self, *args):
        self.params = self.get_params()
        print('INITIALIZING SCRIPT...')
        # for arg in args:
        #     k = arg.split("=")[0]
        #     v = arg.split("=")[1]
        #     self.par[k] = v
        #     print(f"Argument: {k} = {v}")
        self.map(self.get_func, self.params)
        # self.script()
        print('SCRIPT FINISHED!')

    # def get_filename(self, params):
    #     filename = self.name
    #     for par in params:
    #         # if isinstance(par, str):
    #         if isinstance(par, pd.DataFrame):
    #             filename += '__%s__' % par.name
    #         else:
    #             filename += '__%s__' % par
    #     return filename




"""
Author: Diego Pinheiro
github: https://github.com/diegompin

"""

import pandas as pd
import itertools as it
from old.script import AcsPreprocess


def main():
    data_path_acs = 'datalink/acs_wrapper/data_output'
    data_path_spatial = 'datalink/spatial/data_output'
    filepath_zcta = '%s/zcta_county_state.hdf' % data_path_spatial
    readers, year, estimates = AcsPreprocess.get_param()

    df_spatial = pd.read_hdf('%s/zcta_county_state.hdf' % data_path_spatial)

    for (r, y, e) in it.product(readers, year, estimates):
        df_acs = pd.read_csv('%s/%s_%d_%d.csv' % (data_path_acs, r.__name__, y, e))


if __name__ == "__main__":
    main()

# len(df_spatial.ZIP.unique())